# This cog manages user activity - Keeps "active" users in a json and gives them an "active" role
# User activity is defined by (in the last 30 days):
# - Sending messages
# - Joining voice channels

import os
from discord.ext import commands
import logging
import json
import datetime
from discord.ext import tasks

role_id = os.getenv("ACTIVE_ROLE_ID")
if not role_id:
    raise ValueError("ACTIVE_ROLE_ID mnot set")

class ActivityMonitorCog(commands.Cog):
    def __init__(self, bot):
        self.bot = bot
        self.active_users = {} 
        self.activity_file = "active_users.json"
        self.load_active_users()
        logging.info("ActivityMonitorCog initialized")
        self.check_activity.start()

    @tasks.loop(hours=24)
    async def check_activity(self):
        """Check activity daily and update roles"""
        await self.purge_inactive_users()
        logging.info("Completed daily activity check")


    @check_activity.before_loop
    async def before_check_activity(self):
        await self.bot.wait_until_ready()

    def load_active_users(self):
        """Load active users from JSON file"""
        try:
            with open(self.activity_file, 'r') as f:
                data = json.load(f)
                self.active_users = data.get('users', {})
        except FileNotFoundError:
            self.active_users = {}
            logging.info("No existing active users file found, starting fresh")

    def save_active_users(self):
        """Save active users to JSON file"""
        data = {'users': self.active_users}
        with open(self.activity_file, 'w') as f:
            json.dump(data, f)

    async def update(self, user_id, guild_id):
        """Add user to active set and update their role"""
        logging.info(f"Updating activity for user {user_id} in guild {guild_id}")
        current_time = datetime.datetime.now().isoformat()
        self.active_users[str(user_id)] = current_time
        self.save_active_users()
        
        # Get the server and user
        server = self.bot.get_guild(guild_id)
        user = server.get_member(user_id)
        
        if user:
            role = server.get_role(int(role_id))
            if role and role not in user.roles:
                await user.add_roles(role)
                logging.info(f"Added active role to {user.display_name}")

    async def purge_inactive_users(self):
        """Remove active role from users inactive for 30+ days"""
        cutoff_date = datetime.datetime.now() - datetime.timedelta(days=30)
        inactive_users = []
        
        for user_id, last_active in self.active_users.items():
            last_active_date = datetime.datetime.fromisoformat(last_active)
            if last_active_date < cutoff_date:
                inactive_users.append(user_id)
        
        for user_id in inactive_users:
            del self.active_users[user_id]
            # Remove role from all guilds
            for s in self.bot.guilds:
                u = s.get_member(int(user_id))
                if u:
                    role = s.get_role(int(role_id))
                    if role and role in u.roles:
                        await u.remove_roles(role)
                        logging.info(f"Removed active role from {u.display_name} due to inactivity")
        
        if inactive_users:
            self.save_active_users()
        
        # all members in all guilds
        for guild in self.bot.guilds:
            role = guild.get_role(int(role_id))
            if role:
                for member in guild.members:
                    if not member.bot and str(member.id) not in self.active_users:
                        if role in member.roles:
                            await member.remove_roles(role)
                            logging.info(f"Removed active role from {member.display_name} - not in active users list")

    @commands.Cog.listener()
    async def on_message(self, message):
        """Track message activity"""
        if not message.author.bot:
            await self.update(message.author.id, message.guild.id)

    @commands.Cog.listener()
    async def on_voice_state_update(self, member, before, after):
        """Track voice channel activity"""
        if after.channel and not member.bot:
            await self.update(member.id, member.guild.id)

    @commands.Cog.listener()
    async def on_reaction_add(self, reaction, user):
        """Track reaction activity"""
        if not user.bot:
            await self.update(user.id, reaction.message.guild.id)

    @commands.command()
    @commands.has_permissions(administrator=True)
    async def purge_inactive(self, ctx):
        """Manually trigger purge of inactive users"""
        await self.purge_inactive_users()
        await ctx.send("Purged inactive users.")

async def setup(bot):
    await bot.add_cog(ActivityMonitorCog(bot))
