# This cog takes in a /poll <discord_username> and creates a poll in the channel asking whether the user should be invited to the server.
# It takes the active users list and determines how many votes should be reached (50% of active users vote for an option)
# The cog will send an invite link to the user if the poll passes, and if it fails (due to user not having direct messages on) it will send the invite link in the voting channel.
# Invite links are 1 time use only, and will expire after 24 hours.

import os
import discord
from discord.ext import commands
import logging
from datetime import datetime, timedelta
from math import ceil

HOURS = 24

voting_channel_id = int(os.getenv("VOTING_CHANNEL_ID"))
if not voting_channel_id:
    raise ValueError("VOTING_CHANNEL_ID not set")


class PollCog(commands.Cog):
    def __init__(self, bot):
        self.bot = bot
        self.active_polls = {}

    @discord.app_commands.command(
        name="poll", description="Create a poll to invite a user to the server"
    )
    async def poll(self, interaction: discord.Interaction, username: str):
        """Create a poll for inviting a user to the server"""

        # Check if we're in the voting channel
        if interaction.channel_id != voting_channel_id:
            await interaction.response.send_message(
                f"This command can only be used in <#{voting_channel_id}>",
                ephemeral=True,
            )
            return

        active_members = [
            member
            for member in interaction.guild.members
            if not member.bot and member.status != discord.Status.offline
        ]
        required_votes = max(1, ceil(len(active_members) / 2))

        question = f"Should we invite {username} to the server?"
        duration = timedelta(hours=HOURS)

        poll = discord.Poll(question=question, duration=duration)
        poll.add_answer(text="Yes", emoji="👍")
        poll.add_answer(text="No", emoji="👎")

        embed = discord.Embed(
            title=f"Voting for the invitation of {username}",
            description=f"Required votes: **{required_votes}** active members)\n"
            f"Poll will expire in {HOURS} hours.",
            color=discord.Color.blue(),
        )

        await interaction.response.send_message(embed=embed)
        # message = await interaction.original_response()
        # await message.edit(content="", embed=embed, poll=poll)
        # Send separate message with poll
        channel = self.bot.get_channel(voting_channel_id)
        poll_message = await channel.send(
            poll=poll,
        )
        
        self.active_polls[poll_message.id] = {
            "username": username,
            "required_votes": required_votes,
            "channel_id": interaction.channel_id,
            "creator": interaction.user.id,
            "poll": poll,
            "message_id": poll_message.id,
        }
    def __get_msg_id(self, poll: discord.Poll):
        """Get the message ID for a given poll"""
        for msg_id, data in self.active_polls.items():
            if data.get("poll").question == poll.question and data.get("poll").duration == poll.duration:
                return msg_id
        return None

    @commands.Cog.listener()
    async def on_poll_vote_add(self, user: discord.User, answer):
        """Handle poll vote additions"""
        await self._handle_poll_vote_change(answer.poll)

    @commands.Cog.listener()
    async def on_poll_vote_remove(self, user: discord.User, answer):
        """Handle poll vote removals"""
        await self._handle_poll_vote_change(answer.poll)

    async def _handle_poll_vote_change(self, poll: discord.Poll):
        """Handle poll voting updates"""
        message_id = self.__get_msg_id(poll)
        logging.info(f"Handling poll vote change for message ID: {message_id}")
        if not message_id:
            return

        poll_data = self.active_polls[message_id]
        required_votes = poll_data["required_votes"]

        # these are inverted for some reason
        yes_answer = poll.get_answer(1)
        no_answer = poll.get_answer(0)

        yes_count = yes_answer.vote_count if yes_answer else 0
        no_count = no_answer.vote_count if no_answer else 0

        if yes_count >= required_votes:
            await self._end_poll(message_id, poll_data, True)
        elif no_count >= required_votes:
            await self._end_poll(message_id, poll_data, False)


    @commands.Cog.listener()
    async def on_poll_end(self, poll: discord.Poll):
        """When a poll times out"""

        message_id = self.__get_msg_id(poll)
        if not message_id:
            return

        poll_data = self.active_polls[message_id]

        # Determine if poll passed
        yes_answer = poll.get_answer(0)
        no_answer = poll.get_answer(1)

        yes_count = yes_answer.vote_count if yes_answer else 0
        no_count = no_answer.vote_count if no_answer else 0

        passed = yes_count > no_count

        # End the poll
        await self._end_poll(message_id, poll_data, passed)

    def __result_embed(self, username, passed, creator_id):
        """Create result embed for poll completion"""
        title = f"Poll Results for {username}"
        if passed:
            description = "✅"
            color = discord.Color.green()
            if invite_url:
                description += f" <@{creator_id}> The poll passed! Check your DMs for the invite link."
            else:
                description += " Manual invite required"
        else:
            description = "❌ Invitation denied"
            color = discord.Color.red()

        embed = discord.Embed(
            title=title,
            description=description,
            color=color,
            timestamp=datetime.utcnow(),
        )
        embed.set_footer(text="Poll completed")
        return embed
    
    async def _end_poll(self, message_id, poll_data, passed):
        """End the poll and handle the result"""
        username = poll_data["username"]
        channel = self.bot.get_channel(poll_data["channel_id"])
        creator_id = poll_data["creator"]

        if not channel:
            return

        message = await channel.fetch_message(message_id)
        if not message:
            return

        if message.poll and not message.poll.is_finalised():
            try:
                await message.poll.end()
            except Exception as e:
                logging.error(f"Failed to end poll: {e}")

        del self.active_polls[message_id]

        invite_url = None
        if passed:
            # Create invite link to os.env("INVITE_CHANNEL")
            try:
                inv_channel = self.bot.get_channel(int(os.getenv("INVITE_CHANNEL")))
                if not inv_channel:
                    logging.error("INVITE_CHANNEL not found")
                    return
                invite = await inv_channel.create_invite(
                    max_uses=1,
                    max_age=int(timedelta(hours=HOURS).total_seconds()),
                    reason=f"Poll passed for {username}",
                )
                invite_url = invite.url
                
                try:
                    creator = self.bot.get_user(creator_id)
                    if creator:
                        dm_embed = discord.Embed(
                            title=f"Poll Passed for {username}",
                            description=f"The poll you created has passed! Here's the invite link:\n\n{invite_url}",
                            color=discord.Color.green(),
                        )
                        await creator.send(embed=dm_embed)
                    else:
                        logging.error(f"Could not find user with ID {creator_id}")
                except discord.Forbidden:
                    logging.error(f"Could not DM user {creator_id} - DMs may be disabled")
            except discord.Forbidden:
                logging.error("Bot does not have permission to create invites.")
            except Exception as e:
                logging.error(f"Failed to create invite: {e}")
                invite_url = None

        result_embed = self.__result_embed(username, passed, creator_id)
        await message.edit(embed=result_embed)


async def setup(bot):
    await bot.add_cog(PollCog(bot))
